@@ -89,13 +89,14 @@ $(document).ready(function() {
 									Janus.debug("Trying a createOffer too (audio/video sendrecv)");
 									echotest.createOffer(
 										{
-											// No media provided: by default, it's sendrecv for audio and video
-											media: { data: true },	// Let's negotiate data channels as well
-											// If you want to test simulcasting (Chrome and Firefox only), then
-											// pass a ?simulcast=true when opening this demo page: it will turn
-											// the following 'simulcast' property to pass to janus.js to true
-											simulcast: doSimulcast,
-											svc: (vcodec === 'av1' && doSvc) ? doSvc : null,
+											// We want bidirectional audio and video, plus data channels
+											tracks: [
+												{ type: 'audio', capture: true, recv: true },
+												{ type: 'video', capture: true, recv: true,
+													// We may need to enable simulcast or SVC on the video track
+													simulcast: doSimulcast, svc: (vcodec === 'av1' && doSvc) ? doSvc : null },
+												{ type: 'data' },
+											],
 											customizeSdp: function(jsep) {
 												// If DTX is enabled, munge the SDP
 												if(doDtx) {
@@ -284,17 +285,6 @@ $(document).ready(function() {
 									Janus.debug("Remote track (mid=" + mid + ") " + (on ? "added" : "removed") + ":", track);
 									if(!on) {
 										// Track removed, get rid of the stream and the rendering
-										var stream = remoteTracks[mid];
-										if(stream) {
-											try {
-												var tracks = stream.getTracks();
-												for(var i in tracks) {
-													var mst = tracks[i];
-													if(mst)
-														mst.stop();
-												}
-											} catch(e) {}
-										}
 										$('#peervideo' + mid).remove();
 										if(track.kind === "video") {
 											remoteVideos--;
