@@ -126,8 +126,13 @@ $(document).ready(function() {
 										streaming.createAnswer(
 											{
 												jsep: jsep,
-												// We want recvonly audio/video and, if negotiated, datachannels
-												media: { audioSend: false, videoSend: false, data: true },
+												// We only specify data channels here, as this way in
+												// case they were offered we'll enable them. Since we
+												// don't mention audio or video tracks, we autoaccept them
+												// as recvonly (since we won't capture anything ourselves)
+												tracks: [
+													{ type: 'data' }
+												],
 												customizeSdp: function(jsep) {
 													if(stereo && jsep.sdp.indexOf("stereo=1") == -1) {
 														// Make sure that our offer contains stereo too
@@ -154,17 +159,6 @@ $(document).ready(function() {
 										mstreamId = "mstream0";
 									if(!on) {
 										// Track removed, get rid of the stream and the rendering
-										var stream = remoteTracks[mid];
-										if(stream) {
-											try {
-												var tracks = stream.getTracks();
-												for(var i in tracks) {
-													var mst = tracks[i];
-													if(mst)
-														mst.stop();
-												}
-											} catch(e) {}
-										}
 										$('#remotevideo' + mid).remove();
 										if(track.kind === "video") {
 											remoteVideos--;
@@ -182,6 +176,8 @@ $(document).ready(function() {
 										delete remoteTracks[mid];
 										return;
 									}
+									if($('#remotevideo' + mid).length > 0)
+										return;
 									// If we're here, a new track was added
 									var stream = null;
 									if(track.kind === "audio") {
