@@ -149,17 +149,6 @@ $(document).ready(function() {
 									// Now that we're aware of the remote stream, we process it to visualize it
 									if(!on) {
 										// Track removed, get rid of the stream and the rendering
-										var stream = remoteTracks[mid];
-										if(stream) {
-											try {
-												var tracks = stream.getTracks();
-												for(var i in tracks) {
-													var mst = tracks[i];
-													if(mst)
-														mst.stop();
-												}
-											} catch(e) {}
-										}
 										$('#peeraudio' + mid).remove();
 										delete remoteTracks[mid];
 										return;
@@ -317,7 +306,9 @@ function setupWebAudioDemo() {
 		echotest.createOffer(
 			{
 				// We provide our own stream
-				stream: peer.stream,
+				tracks: [
+					{ type: 'audio', capture: peer.stream.getAudioTracks()[0], recv: true }
+				],
 				success: function(jsep) {
 					Janus.debug("Got SDP!", jsep);
 					var body = { audio: true, video: true };
