@@ -1386,8 +1386,14 @@ static void *janus_videocall_handler(void *data) {
 			janus_sdp_find_first_codec(answer, JANUS_SDP_VIDEO, -1, &vcodec);
 			session->vcodec = janus_videocodec_from_name(vcodec);
 			session->opusred_pt = janus_sdp_get_opusred_pt(answer, -1);
-			if(peer)
+			if(peer) {
+				peer->has_audio = session->has_audio;
+				peer->has_video = session->has_video;
+				peer->has_data = session->has_data;
+				peer->acodec = session->acodec;
+				peer->vcodec = session->vcodec;
 				peer->opusred_pt = session->opusred_pt;
+			}
 			if(session->acodec == JANUS_AUDIOCODEC_NONE) {
 				session->has_audio = FALSE;
 				if(peer)
@@ -1491,6 +1497,8 @@ static void *janus_videocall_handler(void *data) {
 				gateway->send_remb(session->handle, session->bitrate ? session->bitrate : 10000000);
 			}
 			janus_videocall_session *peer = session->peer;
+			if(peer)
+				janus_refcount_increase(&peer->ref);
 			if(fallback) {
 				JANUS_LOG(LOG_VERB, "Setting fallback timer (simulcast): %lld (was %"SCNu32")\n",
 					json_integer_value(fallback) ? json_integer_value(fallback) : 250000,
@@ -1543,12 +1551,62 @@ static void *janus_videocall_handler(void *data) {
 						gateway->send_pli(peer->handle);
 				}
 			}
-			if(record) {
-				if(msg_sdp) {
-					session->has_audio = (strstr(msg_sdp, "m=audio") != NULL);
-					session->has_video = (strstr(msg_sdp, "m=video") != NULL);
-					session->has_data = (strstr(msg_sdp, "DTLS/SCTP") != NULL);
+			if(msg_sdp && msg_sdp_type && !strcasecmp(msg_sdp_type, "answer")) {
+				/* Process the answer to see if there were any changes */
+				char error_str[512];
+				janus_sdp *answer = janus_sdp_parse(msg_sdp, error_str, sizeof(error_str));
+				if(answer == NULL) {
+					if(peer)
+						janus_refcount_decrease(&peer->ref);
+					JANUS_LOG(LOG_ERR, "Error parsing answer: %s\n", error_str);
+					error_code = JANUS_VIDEOCALL_ERROR_INVALID_SDP;
+					g_snprintf(error_cause, 512, "Error parsing answer: %s", error_str);
+					goto error;
 				}
+				JANUS_LOG(LOG_VERB, "%s is accepting an update from %s\n", session->username, peer->username);
+				session->has_audio = (strstr(msg_sdp, "m=audio") != NULL);
+				session->has_video = (strstr(msg_sdp, "m=video") != NULL);
+				session->has_data = (strstr(msg_sdp, "DTLS/SCTP") != NULL);
+				/* Check if this user will simulcast */
+				json_t *msg_simulcast = json_object_get(msg->jsep, "simulcast");
+				if(msg_simulcast && janus_get_codec_pt(msg_sdp, "vp8") > 0) {
+					JANUS_LOG(LOG_VERB, "VideoCall callee (%s) cannot do simulcast.\n", session->username);
+				} else {
+					janus_rtp_simulcasting_cleanup(NULL, session->ssrc, session->rid, &session->rid_mutex);
+				}
+				/* Check which codecs we ended up using */
+				const char *acodec = NULL, *vcodec = NULL;
+				janus_sdp_find_first_codec(answer, JANUS_SDP_AUDIO, -1, &acodec);
+				session->acodec = janus_audiocodec_from_name(acodec);
+				janus_sdp_find_first_codec(answer, JANUS_SDP_VIDEO, -1, &vcodec);
+				session->vcodec = janus_videocodec_from_name(vcodec);
+				session->opusred_pt = janus_sdp_get_opusred_pt(answer, -1);
+				janus_videocall_session *peer = session->peer;
+				if(peer) {
+					peer->has_audio = session->has_audio;
+					peer->has_video = session->has_video;
+					peer->has_data = session->has_data;
+					peer->acodec = session->acodec;
+					peer->vcodec = session->vcodec;
+					peer->opusred_pt = session->opusred_pt;
+				}
+				if(session->acodec == JANUS_AUDIOCODEC_NONE) {
+					session->has_audio = FALSE;
+					if(peer)
+						peer->has_audio = FALSE;
+				} else if(peer) {
+					peer->acodec = session->acodec;
+				}
+				if(session->vcodec == JANUS_VIDEOCODEC_NONE) {
+					session->has_video = FALSE;
+					if(peer)
+						peer->has_video = FALSE;
+				} else if(peer) {
+					peer->vcodec = session->vcodec;
+				}
+				janus_sdp_destroy(answer);
+			}
+			if(record) {
 				gboolean recording = json_is_true(record);
 				const char *recording_base = json_string_value(recfile);
 				JANUS_LOG(LOG_VERB, "Recording %s (base filename: %s)\n", recording ? "enabled" : "disabled", recording_base ? recording_base : "not provided");
@@ -1694,6 +1752,8 @@ static void *janus_videocall_handler(void *data) {
 				json_decref(event);
 				json_decref(jsep);
 			}
+			if(peer)
+				janus_refcount_decrease(&peer->ref);
 		} else if(!strcasecmp(request_text, "hangup")) {
 			json_t *hangup = json_object_get(root, "reason");
 			if(hangup && !json_is_string(hangup)) {
