@@ -593,8 +593,10 @@ int janus_pp_webm_process(FILE *file, janus_pp_frame_packet *list, gboolean vp8,
 			/* First we save to the file... */
 			packet->pts = packet->dts = av_rescale_q(tmp->ts-list->ts, timebase, fctx->streams[0]->time_base);
 			if(fctx) {
-				if(av_write_frame(fctx, packet) < 0) {
-					JANUS_LOG(LOG_ERR, "Error writing video frame to file...\n");
+				int res = av_write_frame(fctx, packet);
+				if(res < 0) {
+					JANUS_LOG(LOG_ERR, "Error writing video frame to file... (error %d, %s)\n",
+						res, av_err2str(res));
 				}
 			}
 		}
