@@ -282,7 +282,6 @@ static const char *janus_websockets_reason_string(enum lws_callback_reasons reas
 		CASE_STR(LWS_CALLBACK_CHANGE_MODE_POLL_FD);
 		CASE_STR(LWS_CALLBACK_LOCK_POLL);
 		CASE_STR(LWS_CALLBACK_UNLOCK_POLL);
-		CASE_STR(LWS_CALLBACK_OPENSSL_CONTEXT_REQUIRES_PRIVATE_KEY);
 		CASE_STR(LWS_CALLBACK_USER);
 		CASE_STR(LWS_CALLBACK_RECEIVE_PONG);
 		default:
@@ -1288,20 +1287,23 @@ static int janus_websockets_common_callback(
 			/* Refresh the lws connection validity (avoid sending a ping) */
 			lws_validity_confirmed(ws_client->wsi);
 #endif
+			size_t incoming_length;
 			/* Is this a new message, or part of a fragmented one? */
 			const size_t remaining = lws_remaining_packet_payload(wsi);
 			if(ws_client->incoming == NULL) {
 				JANUS_LOG(LOG_HUGE, "[%s-%p] First fragment: %zu bytes, %zu remaining\n", log_prefix, wsi, len, remaining);
 				ws_client->incoming = g_malloc(len+1);
 				memcpy(ws_client->incoming, in, len);
-				ws_client->incoming[len] = '\0';
+				incoming_length = len;
+				ws_client->incoming[incoming_length] = '\0';
 				JANUS_LOG(LOG_HUGE, "%s\n", ws_client->incoming);
 			} else {
 				size_t offset = strlen(ws_client->incoming);
 				JANUS_LOG(LOG_HUGE, "[%s-%p] Appending fragment: offset %zu, %zu bytes, %zu remaining\n", log_prefix, wsi, offset, len, remaining);
 				ws_client->incoming = g_realloc(ws_client->incoming, offset+len+1);
 				memcpy(ws_client->incoming+offset, in, len);
-				ws_client->incoming[offset+len] = '\0';
+				incoming_length = offset+len;
+				ws_client->incoming[incoming_length] = '\0';
 				JANUS_LOG(LOG_HUGE, "%s\n", ws_client->incoming+offset);
 			}
 			if(remaining > 0 || !lws_is_final_fragment(wsi)) {
@@ -1309,14 +1311,52 @@ static int janus_websockets_common_callback(
 				JANUS_LOG(LOG_HUGE, "[%s-%p] Waiting for more fragments\n", log_prefix, wsi);
 				return 0;
 			}
-			JANUS_LOG(LOG_HUGE, "[%s-%p] Done, parsing message: %zu bytes\n", log_prefix, wsi, strlen(ws_client->incoming));
+			JANUS_LOG(LOG_HUGE, "[%s-%p] Done, parsing message: %zu bytes\n", log_prefix, wsi, incoming_length);
 			/* If we got here, the message is complete: parse the JSON payload */
-			json_error_t error;
-			json_t *root = json_loads(ws_client->incoming, 0, &error);
+			const char *incoming_curr = ws_client->incoming;
+			const char *incoming_end = ws_client->incoming + incoming_length;
+			int message_buffer_count = 0;
+			json_t **message_buffer = NULL;
+			/* Load all JSON messages from the websocket incoming */
+			do {
+				json_error_t error;
+				json_t *message = json_loads(incoming_curr, JSON_DISABLE_EOF_CHECK, &error);
+				if(message != NULL) {
+					/* Position is set to bytes read on success when EOF_CHECK is disabled as above. */
+					incoming_curr += error.position;
+					JANUS_LOG(LOG_HUGE, "[%s-%p] Parsed JSON message - consumed %zu/%zu bytes\n",
+						log_prefix, wsi, (size_t)(incoming_curr - ws_client->incoming), incoming_length);
+					if(incoming_curr == incoming_end) {
+						/* Process messages in order */
+						json_t **msg = message_buffer;
+						json_t **msg_end = message_buffer + message_buffer_count;
+						while(msg != msg_end) {
+							/* Notify the core, no error since we know there weren't any */
+							gateway->incoming_request(&janus_websockets_transport, ws_client->ts, NULL, admin, *msg++, NULL);
+						}
+						/* Notify the core, no error since we know there weren't any */
+						gateway->incoming_request(&janus_websockets_transport, ws_client->ts, NULL, admin, message, NULL);
+						break;
+					} else {
+						/* Buffer the message */
+						message_buffer = (json_t**)g_realloc(message_buffer, sizeof(json_t*) * (message_buffer_count + 1));
+						message_buffer[message_buffer_count++] = message;
+					}
+				} else {
+					/* Release any buffered messages */
+					json_t **msg = message_buffer;
+					json_t **msg_end = message_buffer + message_buffer_count;
+					while(msg != msg_end) {
+						json_decref(*msg++);
+					}
+					/* Notify the core, passing the error since we have no message */
+					gateway->incoming_request(&janus_websockets_transport, ws_client->ts, NULL, admin, NULL, &error);
+					break;
+				}
+			} while(incoming_curr < incoming_end);
+			g_free(message_buffer);
 			g_free(ws_client->incoming);
 			ws_client->incoming = NULL;
-			/* Notify the core, passing both the object and, since it may be needed, the error */
-			gateway->incoming_request(&janus_websockets_transport, ws_client->ts, NULL, admin, root, &error);
 			return 0;
 		}
 #if (LWS_LIBRARY_VERSION_MAJOR >= 3)
